/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "AnalogueRepeaterConfigApp.h"
#include "Version.h"

#include <wx/cmdline.h>

IMPLEMENT_APP(CAnalogueRepeaterConfigApp)

const wxChar* NAME_PARAM = wxT("Repeater Name");


CAnalogueRepeaterConfigApp::CAnalogueRepeaterConfigApp() :
wxApp(),
m_name(),
m_frame(NULL),
m_config(NULL)
{
}

CAnalogueRepeaterConfigApp::~CAnalogueRepeaterConfigApp()
{
}

bool CAnalogueRepeaterConfigApp::OnInit()
{
	SetVendorName(VENDOR_NAME);

	if (!wxApp::OnInit())
		return false;

	m_config = new CAnalogueRepeaterConfig(new wxConfig(APPLICATION_NAME), m_name);

	wxString frameName = APPLICATION_NAME + wxT(" - ");
	if (!m_name.IsEmpty()) {
		frameName.Append(m_name);
		frameName.Append(wxT(" - "));
	}
	frameName.Append(VERSION);

	m_frame = new CAnalogueRepeaterConfigFrame(frameName);
	m_frame->Show();

	SetTopWindow(m_frame);

	return wxApp::OnInit();
}

int CAnalogueRepeaterConfigApp::OnExit()
{
	delete m_config;

	return 0;
}

void CAnalogueRepeaterConfigApp::OnInitCmdLine(wxCmdLineParser& parser)
{
	parser.AddParam(NAME_PARAM, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);

	wxApp::OnInitCmdLine(parser);
}

bool CAnalogueRepeaterConfigApp::OnCmdLineParsed(wxCmdLineParser& parser)
{
	if (!wxApp::OnCmdLineParsed(parser))
		return false;

	if (parser.GetParamCount() > 0U)
		m_name = parser.GetParam(0);

	return true;
}

void CAnalogueRepeaterConfigApp::getCallsign(wxString& callsign, wxString& beacon, unsigned int& speed, unsigned int& freq, wxFloat32& level1, wxFloat32& level2) const
{
	m_config->getCallsign(callsign, beacon, speed, freq, level1, level2);
}

void CAnalogueRepeaterConfigApp::setCallsign(const wxString& callsign, const wxString& beacon, unsigned int speed, unsigned int freq, wxFloat32 level1, wxFloat32 level2) const
{
	m_config->setCallsign(callsign, beacon, speed, freq, level1, level2);
}

void CAnalogueRepeaterConfigApp::getAck(wxString& radioAck, wxString& extAck, unsigned int& speed, unsigned int& freq, wxFloat32& level, unsigned int& ack, unsigned int& minimum) const
{
	m_config->getAck(radioAck, extAck, speed, freq, level, ack, minimum);
}

void CAnalogueRepeaterConfigApp::setAck(const wxString& radioAck, const wxString& extAck, unsigned int speed, unsigned int freq, wxFloat32 level, unsigned int ack, unsigned int minimum) const
{
	m_config->setAck(radioAck, extAck, speed, freq, level, ack, minimum);
}

void CAnalogueRepeaterConfigApp::getTimes(unsigned int& callsignTime, unsigned int& timeout, unsigned int& lockoutTime, unsigned int& hangTime, unsigned int& latchTime) const
{
	m_config->getTimes(callsignTime, timeout, lockoutTime, hangTime, latchTime);
}

void CAnalogueRepeaterConfigApp::setTimes(unsigned int callsignTime, unsigned int timeout, unsigned int lockoutTime, unsigned int hangTime, unsigned int latchTime) const
{
	m_config->setTimes(callsignTime, timeout, lockoutTime, hangTime, latchTime);
}

void CAnalogueRepeaterConfigApp::getTones(bool& tbEnable, wxFloat32& tbThreshold, wxFloat32& ctcssFreq, bool& ctcssInternal, wxFloat32& ctcssThresh, wxFloat32& ctcssLevel, ANALOGUE_CTCSS_OUTPUT& ctcssOutput) const
{
	m_config->getTones(tbEnable, tbThreshold, ctcssFreq, ctcssInternal, ctcssThresh, ctcssLevel, ctcssOutput);
}

void CAnalogueRepeaterConfigApp::setTones(bool tbEnable, wxFloat32 tbThreshold, wxFloat32 ctcssFreq, bool ctcssInternal, wxFloat32 ctcssThresh, wxFloat32 ctcssLevel, ANALOGUE_CTCSS_OUTPUT ctcssOutput) const
{
	m_config->setTones(tbEnable, tbThreshold, ctcssFreq, ctcssInternal, ctcssThresh, ctcssLevel, ctcssOutput);
}

void CAnalogueRepeaterConfigApp::getFeel(ANALOGUE_CALLSIGN_START& callsignAtStart, bool& callsignAtEnd, ANALOGUE_TIMEOUT_TYPE& timeoutType, ANALOGUE_CALLSIGN_HOLDOFF& callsignHoldoff) const
{
	m_config->getFeel(callsignAtStart, callsignAtEnd, timeoutType, callsignHoldoff);
}

void CAnalogueRepeaterConfigApp::setFeel(ANALOGUE_CALLSIGN_START callsignAtStart, bool callsignAtEnd, ANALOGUE_TIMEOUT_TYPE timeoutType, ANALOGUE_CALLSIGN_HOLDOFF callsignHoldoff) const
{
	m_config->setFeel(callsignAtStart, callsignAtEnd, timeoutType, callsignHoldoff);
}

void CAnalogueRepeaterConfigApp::getRadio(wxString& readDevice, wxString& writeDevice, unsigned int& delay, bool& deEmphasis, bool& preEmphasis, bool& vogad) const
{
	m_config->getRadio(readDevice, writeDevice, delay, deEmphasis, preEmphasis, vogad);
}

void CAnalogueRepeaterConfigApp::setRadio(const wxString& readDevice, const wxString& writeDevice, unsigned int delay, bool deEmphasis, bool preEmphasis, bool vogad) const
{
	m_config->setRadio(readDevice, writeDevice, delay, deEmphasis, preEmphasis, vogad);
}

void CAnalogueRepeaterConfigApp::getExternal(ANALOGUE_EXTERNAL_MODE& mode, wxString& readDevice, wxString& writeDevice, bool& vogad, wxString& device, SERIALPIN& txPin, SERIALPIN& rxPin) const
{
	m_config->getExternal(mode, readDevice, writeDevice, vogad, device, txPin, rxPin);
}

void CAnalogueRepeaterConfigApp::setExternal(ANALOGUE_EXTERNAL_MODE mode, const wxString& readDevice, const wxString& writeDevice, bool vogad, const wxString& device, SERIALPIN txPin, SERIALPIN rxPin) const
{
	m_config->setExternal(mode, readDevice, writeDevice, vogad, device, txPin, rxPin);
}

void CAnalogueRepeaterConfigApp::getController(wxString& type, unsigned int& pttDelay, bool& pttInvert, bool& squelchInvert) const
{
	m_config->getController(type, pttDelay, pttInvert, squelchInvert);
}

void CAnalogueRepeaterConfigApp::setController(const wxString& type, unsigned int pttDelay, bool pttInvert, bool squelchInvert) const
{
	m_config->setController(type, pttDelay, pttInvert, squelchInvert);
}

void CAnalogueRepeaterConfigApp::getDTMF(bool& radio, bool& external, wxString& shutdown, wxString& startup, wxString& timeout, wxString& timeReset, wxString& command1, wxString& command1Line, wxString& command2, wxString& command2Line ,wxString& output1, wxString& output2, wxString& output3, wxString& output4, wxFloat32& threshold) const
{
	m_config->getDTMF(radio, external, shutdown, startup, timeout, timeReset, command1, command1Line, command2, command2Line, output1, output2, output3, output4, threshold);
}

void CAnalogueRepeaterConfigApp::setDTMF(bool radio, bool external, const wxString& shutdown, const wxString& startup, const wxString& timeout, const wxString& timeReset, const wxString& command1, const wxString& command1Line, const wxString& command2, const wxString& command2Line, const wxString& output1, const wxString& output2, const wxString& output3, const wxString& output4, wxFloat32 threshold) const
{
	m_config->setDTMF(radio, external, shutdown, startup, timeout, timeReset, command1, command1Line, command2, command2Line, output1, output2, output3, output4, threshold);
}

void CAnalogueRepeaterConfigApp::getActiveHang(unsigned int& time) const
{
	m_config->getActiveHang(time);
}

void CAnalogueRepeaterConfigApp::setActiveHang(unsigned int time) const
{
	m_config->setActiveHang(time);
}
