/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "AnalogueRepeaterConfigCallsignSet.h"

const unsigned int ID_WIDTH    = 300U;

const unsigned int BORDER_SIZE = 5U;

CAnalogueRepeaterConfigCallsignSet::CAnalogueRepeaterConfigCallsignSet(wxWindow* parent, int id, const wxString& title, const wxString& callsign, const wxString& beacon, unsigned int speed, unsigned int freq, wxFloat32 level1, wxFloat32 level2) :
wxPanel(parent, id),
m_title(title),
m_callsign(NULL),
m_beacon(NULL),
m_speed(NULL),
m_freq(NULL),
m_level1(NULL),
m_level2(NULL)
{
	wxFlexGridSizer* sizer = new wxFlexGridSizer(2);

	wxStaticText* callsignLabel = new wxStaticText(this, -1, _("Callsign"));
	sizer->Add(callsignLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_callsign = new CCWTextCtrl(this, -1, callsign, wxDefaultPosition, wxSize(ID_WIDTH, -1));
	sizer->Add(m_callsign, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* beaconLabel = new wxStaticText(this, -1, _("Beacon"));
	sizer->Add(beaconLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_beacon = new CCWTextCtrl(this, -1, beacon, wxDefaultPosition, wxSize(ID_WIDTH, -1));
	sizer->Add(m_beacon, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* speedLabel = new wxStaticText(this, -1, _("Speed (WPM)"));
	sizer->Add(speedLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_speed = new wxSlider(this, -1, speed, 8, 25, wxDefaultPosition, wxSize(ID_WIDTH, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_speed, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* freqLabel = new wxStaticText(this, -1, _("Frequency (Hz)"));
	sizer->Add(freqLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_freq = new wxSlider(this, -1, freq, 500, 2000, wxDefaultPosition, wxSize(ID_WIDTH, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_freq, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* level1Label = new wxStaticText(this, -1, _("Level 1 (%)"));
	sizer->Add(level1Label, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_level1 = new wxSlider(this, -1, int(level1 * 100.0F + 0.5F), 0, 100, wxDefaultPosition, wxSize(ID_WIDTH, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_level1, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* level2Label = new wxStaticText(this, -1, _("Level 2 (%)"));
	sizer->Add(level2Label, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_level2 = new wxSlider(this, -1, int(level2 * 100.0F + 0.5F), 0, 100, wxDefaultPosition, wxSize(ID_WIDTH, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_level2, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	SetAutoLayout(true);

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}


CAnalogueRepeaterConfigCallsignSet::~CAnalogueRepeaterConfigCallsignSet()
{
}

bool CAnalogueRepeaterConfigCallsignSet::Validate()
{
	bool res = !getCallsign().IsEmpty();
	if (!res) {
		wxMessageDialog dialog(this, _("The Callsign may not be empty"), m_title + _(" Error"), wxICON_ERROR);
		dialog.ShowModal();
	}

	res = !getBeacon().IsEmpty();
	if (!res) {
		wxMessageDialog dialog(this, _("The Beacon may not be empty"), m_title + _(" Error"), wxICON_ERROR);
		dialog.ShowModal();
	}

	return res;
}

wxString CAnalogueRepeaterConfigCallsignSet::getCallsign() const
{
	return m_callsign->GetValue().MakeUpper();
}

wxString CAnalogueRepeaterConfigCallsignSet::getBeacon() const
{
	return m_beacon->GetValue().MakeUpper();
}

unsigned int CAnalogueRepeaterConfigCallsignSet::getSpeed() const
{
	return m_speed->GetValue();
}

unsigned int CAnalogueRepeaterConfigCallsignSet::getFreq() const
{
	return m_freq->GetValue();
}

wxFloat32 CAnalogueRepeaterConfigCallsignSet::getLevel1() const
{
	return wxFloat32(m_level1->GetValue()) / 100.0F;
}

wxFloat32 CAnalogueRepeaterConfigCallsignSet::getLevel2() const
{
	return wxFloat32(m_level2->GetValue()) / 100.0F;
}
