/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "DStarRepeaterConfigFrame.h"
#include "DStarRepeaterConfigApp.h"
#include "DStarDefines.h"
#include "Version.h"

const unsigned int BORDER_SIZE   = 5U;

#include <wx/gbsizer.h>
#include <wx/aboutdlg.h>
#include <wx/notebook.h>

BEGIN_EVENT_TABLE(CDStarRepeaterConfigFrame, wxFrame)
	EVT_MENU(wxID_EXIT, CDStarRepeaterConfigFrame::onQuit)
	EVT_MENU(wxID_ABOUT, CDStarRepeaterConfigFrame::onAbout)

	EVT_CLOSE(CDStarRepeaterConfigFrame::onClose)
END_EVENT_TABLE()

CDStarRepeaterConfigFrame::CDStarRepeaterConfigFrame(const wxString& title) :
wxFrame(NULL, -1, title),
m_callsign(NULL),
m_network(NULL),
m_times(NULL),
m_radio(NULL),
m_controller(NULL),
m_control1(NULL),
m_control2(NULL),
m_active(NULL)
{
	SetMenuBar(createMenuBar());

	wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);

	wxPanel* panel = new wxPanel(this);

	wxBoxSizer* panelSizer = new wxBoxSizer(wxVERTICAL);

	wxNotebook* noteBook = new wxNotebook(panel, -1);

	wxString callsign, gateway;
	DSTAR_MODE mode;
	bool ack;
	::wxGetApp().getCallsign(callsign, gateway, mode, ack);

	wxString gatewayAddress, localAddress;
	unsigned int gatewayPort, localPort, networkDelay;
	::wxGetApp().getNetwork(gatewayAddress, gatewayPort, localAddress, localPort, networkDelay);

	unsigned int timeout, ackTime;
	::wxGetApp().getTimes(timeout, ackTime);

	wxString readDevice, writeDevice;
	bool rxInvert, txInvert;
	wxFloat32 level;
	::wxGetApp().getRadio(readDevice, writeDevice, level, rxInvert, txInvert);

	wxString interfaceType;
	int pttDelay;
	bool pttInvert;
	::wxGetApp().getController(interfaceType, pttDelay, pttInvert);

	bool enabled;
	wxString rpt1Callsign, rpt2Callsign;
	wxString shutdown, startup;
	wxString command1, command1Line;
	wxString command2, command2Line;
	wxString output1, output2, output3, output4;
	::wxGetApp().getControl(enabled, rpt1Callsign, rpt2Callsign, shutdown, startup, command1, command1Line, command2, command2Line, output1, output2, output3, output4);

	unsigned int hangTime;
	::wxGetApp().getActiveHang(hangTime);

	m_callsign = new CDStarRepeaterConfigCallsignSet(noteBook, -1, APPLICATION_NAME, callsign, gateway, mode, ack);
	noteBook->AddPage(m_callsign, _("Callsign"), true);

	m_network = new CDStarRepeaterConfigNetworkSet(noteBook, -1, APPLICATION_NAME, gatewayAddress, gatewayPort, localAddress, localPort, networkDelay);
	noteBook->AddPage(m_network, _("Network"), false);

	m_times = new CDStarRepeaterConfigTimesSet(noteBook, -1, APPLICATION_NAME, timeout, ackTime);
	noteBook->AddPage(m_times, _("Timers"), false);

	m_radio = new CDStarRepeaterConfigRadioSet(noteBook, -1, APPLICATION_NAME, readDevice, writeDevice, level, rxInvert, txInvert);
	noteBook->AddPage(m_radio, _("Radio"), false);

	m_controller = new CDStarRepeaterConfigControllerSet(noteBook, -1, APPLICATION_NAME, interfaceType, pttDelay, pttInvert);
	noteBook->AddPage(m_controller, _("Controller"), false);

	m_control1 = new CDStarRepeaterConfigControl1Set(noteBook, -1, APPLICATION_NAME, enabled, rpt1Callsign, rpt2Callsign, shutdown, startup, output1, output2, output3, output4);
	noteBook->AddPage(m_control1, _("Control 1"), false);

	m_control2 = new CDStarRepeaterConfigControl2Set(noteBook, -1, APPLICATION_NAME, command1, command1Line, command2, command2Line);
	noteBook->AddPage(m_control2, _("Control 2"), false);

	m_active = new CActiveHangSet(noteBook, -1, APPLICATION_NAME, hangTime);
	noteBook->AddPage(m_active, _("Active Output"), false);

	panelSizer->Add(noteBook, 1, wxALL | wxGROW, BORDER_SIZE);

	panel->SetSizer(panelSizer);
	panelSizer->SetSizeHints(panel);

	mainSizer->Add(panel);

	SetSizer(mainSizer);
	mainSizer->SetSizeHints(this);
}

CDStarRepeaterConfigFrame::~CDStarRepeaterConfigFrame()
{
}

wxMenuBar* CDStarRepeaterConfigFrame::createMenuBar()
{
	wxMenu* fileMenu = new wxMenu();
	fileMenu->Append(wxID_EXIT, _("Exit"));

	wxMenu* helpMenu = new wxMenu();
	helpMenu->Append(wxID_ABOUT, _("About D-Star Repeater Config"));

	wxMenuBar* menuBar = new wxMenuBar();
	menuBar->Append(fileMenu, _("File"));
	menuBar->Append(helpMenu, _("Help"));

	return menuBar;
}

void CDStarRepeaterConfigFrame::onQuit(wxCommandEvent& event)
{
	Close(true);
}

void CDStarRepeaterConfigFrame::onClose(wxCloseEvent& event)
{
	wxMessageDialog dialog(this, _("Save changes?"), _("D-Star Repeater Config"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION);
	int id = dialog.ShowModal();

	if (id == wxID_NO)
		Destroy();

	if (!m_callsign->Validate() || !m_network->Validate() || !m_times->Validate() || !m_radio->Validate() || !m_controller->Validate() || !m_control1->Validate() || !m_control2->Validate()) {
		event.Veto(true);
		return;
	}

	saveConfig();

	Destroy();
}

void CDStarRepeaterConfigFrame::onAbout(wxCommandEvent& event)
{
	wxAboutDialogInfo info;
	info.AddDeveloper(wxT("Jonathan Naylor, G4KLX"));
	info.SetCopyright(wxT("(C) 2009,2010 using GPL v2 or later"));
	info.SetName(APPLICATION_NAME);
	info.SetVersion(VERSION);
	info.SetDescription(_("This program configures the D-Star repeater."));

	::wxAboutBox(info);
}

void CDStarRepeaterConfigFrame::saveConfig()
{
	wxString callsign = m_callsign->getCallsign();
	wxString gateway  = m_callsign->getGateway();
	DSTAR_MODE mode   = m_callsign->getMode();
	bool ack          = m_callsign->getAck();
	::wxGetApp().setCallsign(callsign, gateway, mode, ack);

	wxString gatewayAddress   = m_network->getGatewayAddress();
	unsigned int gatewayPort  = m_network->getGatewayPort();
	wxString localAddress     = m_network->getLocalAddress();
	unsigned int localPort    = m_network->getLocalPort();
	unsigned int networkDelay = m_network->getNetworkDelay();
	::wxGetApp().setNetwork(gatewayAddress, gatewayPort, localAddress, localPort, networkDelay);

	unsigned int timeout = m_times->getTimeout();
	unsigned int ackTime = m_times->getAckTime();
	::wxGetApp().setTimes(timeout, ackTime);

	wxString readDevice     = m_radio->getReadDevice();
	wxString writeDevice    = m_radio->getWriteDevice();
	bool rxInvert           = m_radio->getRXInvert();
	bool txInvert           = m_radio->getTXInvert();
	wxFloat32 level         = m_radio->getLevel();
	::wxGetApp().setRadio(readDevice, writeDevice, level, rxInvert, txInvert);

	wxString interfaceType = m_controller->getType();
	int pttDelay           = m_controller->getPTTDelay();
	bool pttInvert         = m_controller->getPTTInvert();
	::wxGetApp().setController(interfaceType, pttDelay, pttInvert);

	bool enabled          = m_control1->getEnabled();
	wxString rpt1Callsign = m_control1->getRPT1Callsign();
	wxString rpt2Callsign = m_control1->getRPT2Callsign();
	wxString shutdown     = m_control1->getShutdown();
	wxString startup      = m_control1->getStartup();
	wxString output1      = m_control1->getOutput1();
	wxString output2      = m_control1->getOutput2();
	wxString output3      = m_control1->getOutput3();
	wxString output4      = m_control1->getOutput4();
	wxString command1     = m_control2->getCommand1();
	wxString command1Line = m_control2->getCommand1Line();
	wxString command2     = m_control2->getCommand2();
	wxString command2Line = m_control2->getCommand2Line();
	::wxGetApp().setControl(enabled, rpt1Callsign, rpt2Callsign, shutdown, startup, command1, command1Line, command2, command2Line, output1, output2, output3, output4);

	unsigned int hangTime = m_active->getTime();
	::wxGetApp().setActiveHang(hangTime);
}
