/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "DStarRepeaterConfigRadioSet.h"

#include "SoundCardReaderWriter.h"

const unsigned int BORDER_SIZE    = 5U;
const unsigned int CONTROL_WIDTH1 = 300U;
const unsigned int CONTROL_WIDTH2 = 100U;

CDStarRepeaterConfigRadioSet::CDStarRepeaterConfigRadioSet(wxWindow* parent, int id, const wxString& title, const wxString& readDevice, const wxString& writeDevice, wxFloat32 level, bool rxInvert, bool txInvert) :
wxPanel(parent, id),
m_title(title),
m_readDevice(NULL),
m_writeDevice(NULL),
m_level(NULL),
m_rxInvert(NULL),
m_txInvert(NULL)
{
	wxFlexGridSizer* sizer = new wxFlexGridSizer(2);

	wxArrayString readDevices = CSoundCardReaderWriter::getReadDevices();
	wxArrayString writeDevices = CSoundCardReaderWriter::getWriteDevices();

	wxStaticText* readLabel = new wxStaticText(this, -1, _("Input"));
	sizer->Add(readLabel, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	m_readDevice = new wxChoice(this, -1, wxDefaultPosition, wxSize(CONTROL_WIDTH1, -1), readDevices);
	sizer->Add(m_readDevice, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* writeLabel = new wxStaticText(this, -1, _("Output"));
	sizer->Add(writeLabel, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	m_writeDevice = new wxChoice(this, -1, wxDefaultPosition, wxSize(CONTROL_WIDTH1, -1), writeDevices);
	sizer->Add(m_writeDevice, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* levelLabel = new wxStaticText(this, -1, _("Level (%)"));
	sizer->Add(levelLabel, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	m_level = new wxSlider(this, -1, int(level * 100.0F + 0.5F), 0, 100, wxDefaultPosition, wxSize(CONTROL_WIDTH1, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_level, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* rxInvertLabel = new wxStaticText(this, -1, _("RX Inversion"));
	sizer->Add(rxInvertLabel, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	m_rxInvert = new wxChoice(this, -1, wxDefaultPosition, wxSize(CONTROL_WIDTH2, -1));
	m_rxInvert->Append(_("Off"));
	m_rxInvert->Append(_("On"));
	sizer->Add(m_rxInvert, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* txInvertLabel = new wxStaticText(this, -1, _("TX Inversion"));
	sizer->Add(txInvertLabel, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	m_txInvert = new wxChoice(this, -1, wxDefaultPosition, wxSize(CONTROL_WIDTH2, -1));
	m_txInvert->Append(_("Off"));
	m_txInvert->Append(_("On"));
	sizer->Add(m_txInvert, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	if (readDevices.GetCount() == 0U) {
		m_readDevice->Append(_("None"));
		m_readDevice->SetSelection(0);
	} else {
		bool res = m_readDevice->SetStringSelection(readDevice);
		if (!res)
			m_readDevice->SetSelection(0);
	}

	if (writeDevices.GetCount() == 0U) {
		m_writeDevice->Append(_("None"));
		m_writeDevice->SetSelection(0);
	} else {
		bool res = m_writeDevice->SetStringSelection(writeDevice);
		if (!res)
			m_writeDevice->SetSelection(0);
	}

	m_rxInvert->SetSelection(rxInvert ? 1 : 0);
	m_txInvert->SetSelection(txInvert ? 1 : 0);

	SetAutoLayout(true);

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}


CDStarRepeaterConfigRadioSet::~CDStarRepeaterConfigRadioSet()
{
}

bool CDStarRepeaterConfigRadioSet::Validate()
{
	if (m_readDevice->GetCurrentSelection() == wxNOT_FOUND)
		return false;

	if (m_writeDevice->GetCurrentSelection() == wxNOT_FOUND)
		return false;

	if (m_rxInvert->GetCurrentSelection() == wxNOT_FOUND)
		return false;

	return m_txInvert->GetCurrentSelection() != wxNOT_FOUND;
}

wxString CDStarRepeaterConfigRadioSet::getReadDevice() const
{
	wxString device = m_readDevice->GetStringSelection();

	if (device.IsSameAs(_("None")))
		return wxEmptyString;
	else
		return device;
}

wxString CDStarRepeaterConfigRadioSet::getWriteDevice() const
{
	wxString device = m_writeDevice->GetStringSelection();

	if (device.IsSameAs(_("None")))
		return wxEmptyString;
	else
		return device;
}

wxFloat32 CDStarRepeaterConfigRadioSet::getLevel() const
{
	int n = m_level->GetValue();

	return wxFloat32(n) / 100.0F;
}

bool CDStarRepeaterConfigRadioSet::getRXInvert() const
{
	int n = m_rxInvert->GetCurrentSelection();

	switch (n) {
		case 0:
			return false;
		case 1:
			return true;
		default:
			return false;
	}
}

bool CDStarRepeaterConfigRadioSet::getTXInvert() const
{
	int n = m_txInvert->GetCurrentSelection();

	switch (n) {
		case 0:
			return false;
		case 1:
			return true;
		default:
			return false;
	}
}
