/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ParrotControllerConfig.h"
#include "ParrotControllerD.h"
#include "Version.h"
#include "Logger.h"

#include <wx/cmdline.h>
#include <wx/wfstream.h>
#include <wx/fileconf.h>
#include <wx/filename.h>

const wxChar* NAME_PARAM       = wxT("Repeater Name");
const wxChar* NOLOGGING_SWITCH = wxT("nolog");

int main(int argc, char** argv)
{
	bool res = ::wxInitialize();
	if (!res) {
		::fprintf(stderr, "parrotcontrollerd: failed to initialise the wxWidgets library, exiting\n");
		return -1;
	}

	wxCmdLineParser parser(argc, argv);
	parser.AddSwitch(NOLOGGING_SWITCH);
	parser.AddParam(NAME_PARAM);

	int cmd = parser.Parse();
	if (cmd != 0) {
		::wxUninitialize();
		return 0;
	}

	bool nolog = parser.Found(NOLOGGING_SWITCH);

	wxString name;
	if (parser.GetParamCount() > 0U)
		name = parser.GetParam(0U);

	CParrotControllerD repeater(nolog, name);

	if (!repeater.init()) {
		::wxUninitialize();
		return 1;
	}

	repeater.run();

	::wxUninitialize();
	return 0;
}

CParrotControllerD::CParrotControllerD(bool nolog, const wxString& name) :
m_name(name),
m_nolog(nolog),
m_thread(NULL)
{
}

CParrotControllerD::~CParrotControllerD()
{
}

bool CParrotControllerD::init()
{
	if (!m_nolog) {
		wxString logBaseName = LOG_BASE_NAME;
		if (!m_name.IsEmpty()) {
			logBaseName.Append(wxT("_"));
			logBaseName.Append(m_name);
		}

		wxFileName loggerName(wxFileName::GetHomeDir(), logBaseName, wxT("log"));
		wxLog* log = new CLogger(loggerName.GetFullPath());
		wxLog::SetActiveTarget(log);
	} else {
		wxLog::SetActiveTarget(new wxLogStderr);
	}

	::wxLogInfo(wxT("Starting ") + APPLICATION_NAME + wxT(" - ") + VERSION);

	return createThread();
}

void CParrotControllerD::run()
{
	m_thread->Create();
	m_thread->Run();

	m_thread->Wait();
	delete m_thread;

	::wxLogInfo(APPLICATION_NAME + wxT(" is exiting"));
}

bool CParrotControllerD::createThread()
{
	wxFileName fileName(wxFileName::GetHomeDir(), CONFIG_FILE_NAME);
	wxFileInputStream stream(fileName.GetFullPath());
	if (!stream.IsOk()) {
		::wxLogError(wxT("Cannot open the config file - %s"), fileName.GetFullPath().c_str());
		return false;
	}

	CParrotControllerConfig config(new wxFileConfig(stream), m_name);

	m_thread = new CParrotControllerThread;

	PARROT_MODE mode;
	unsigned int beaconTime, turnaroundTime;
	wxString beaconFileName;
	bool keepFile;
	config.getMode(mode, beaconTime, beaconFileName, turnaroundTime, keepFile);
	m_thread->setMode(mode, beaconTime, beaconFileName, turnaroundTime, keepFile);
	::wxLogInfo(wxT("Mode set to %d, beacon time set to %u secs, beacon file name set to %s, turnaround time set to %u secs, keep file set to %d"), mode, beaconTime, beaconFileName.c_str(), turnaroundTime, keepFile);

	wxString repeaterAddress, localAddress;
	unsigned int repeaterPort, localPort;
	config.getNetwork(repeaterAddress, repeaterPort, localAddress, localPort);
	::wxLogInfo(wxT("Repeater set to %s:%u, local set to %s:%u"), repeaterAddress.c_str(), repeaterPort, localAddress.c_str(), localPort);

	if (!repeaterAddress.IsEmpty())
		m_thread->setNetwork(repeaterAddress, repeaterPort, localAddress, localPort);

	return true;
}

